@@ -17,7 +17,7 @@ from .electrochem import *
 import sklearn         
 from sklearn import linear_model, datasets
 from sklearn.model_selection import train_test_split
-from sklearn.metrics import mean_squared_error, r2_score
+from sklearn.metrics import mean_squared_error, r2_score,mean_absolute_percentage_error
 # Note - you will need version 0.24.1 of scikit-learn to load this library (SequentialFeatureSelector)
 from sklearn.feature_selection import f_regression, SequentialFeatureSelector
 from sklearn.preprocessing import PolynomialFeatures
@@ -257,11 +257,20 @@ def ml_features(eis_data,cycling_data):
     data=np.hstack((var[:],freq_fit_max,Z1max,Z2max,Z1_high_f,Z2_high_f))
     data=np.transpose(data)
     return(data.reshape(1,-1))
-def average_error(y_test, y_train):
-    y_diff = np.subtract(y_test, y_train)
+
+def average_error(y_test, y_pred):
+    y_diff = np.subtract(y_test, y_pred)
     y_diff_abs = np.abs(y_diff)
     average_error = np.sum(y_diff_abs)/y_diff_abs.size
     return (round(average_error,2))
+def average_err_per_cyc(y_test,y_pred):
+    n=len(y_pred[0])
+    error=np.zeros(n)
+    for i in range(0,n):
+        error[i]=average_error(y_test[:,i],y_pred[:,i])
+    return(error)
+
+        
 
 
 
@@ -285,13 +294,10 @@ def rdf_online(eis_data,cycling_data,n_estimators, max_features, max_depth,test_
     r2=model.score(X_test,y_test)
     features= ml_features(eis_data,cycling_data)
     cap_ret=model.predict(features)
-    fig, ax = plt.subplots()
-    fig.text(0.45,0.8,'Model $R^{2}$ = '+str(round(r2,2)))
-    fig=plt.plot(cap_ret[0])
-    plt.xlabel('Cycle Number')
-    plt.ylabel('Capacity Retetion, [%]')
+    rel_err=mean_absolute_percentage_error(model.predict(X_test),y_test)
+    
 
-    return (fig,round(cap_ret[0][cyc_num-1],2),average_error(y_test,model.predict(X_test)))
+    return (round(cap_ret[0][cyc_num-1],2),round(cap_ret[0][cyc_num-1]*rel_err,2))
 
 
 def battery_clf(eis_data,cycling_data):
@@ -305,8 +311,18 @@ def battery_clf(eis_data,cycling_data):
     gdb_pred=gdb.predict(X_features)
     average_error= 0.15
     result = np.mean([rdf_pred[0][299],nn_pred[0][299],gdb_pred[0]])
-    clf=str(math.floor(result-average_error))
-    return (Grade[clf],round(result,2),average_error)
+    if (result-average_error)<93:
+        return('Bad Cell',round(result,2),average_error)
+    if (result-average_error)>98:
+        return('Top Graded Cell',round(result,2),average_error)
+    if 97<(result-average_error)<98:
+        return('A',round(result,2),average_error)
+    if 95<(result-average_error)<97:
+        return('B',round(result,2),average_error)
+    if 93<(result-average_error)<97:
+        return('C',round(result,2),average_error)
+
+
 
 
 def gbr_online(eis_data,cycling_data,n_estimators, max_depth, learning_rate,test_size,cyc_num):
@@ -433,13 +449,10 @@ def cap_ret_online_NN(eis_data,cycling_data, hidden_layers_sizes, learning_rate,
     #Predict from given data
     
     cap_ret_pred = online_NN.predict(input_features)
-    fig, ax = plt.subplots()
-    fig.text(0.45,0.8,'Model $R^{2}$ = '+str(round(r2_online_NN,2)))
-    fig=plt.plot(cap_ret_pred[0])
-    plt.xlabel('Cycle Number')
-    plt.ylabel('Capacity Retetion, [%]')
-    avg_error=average_error(y_test,online_NN.predict(X_test))
-    return (fig,round(cap_ret_pred[0][cyc_num-1],2),avg_error)
+    
+    rel_err=mean_absolute_percentage_error(model.predict(X_test),y_test)
+
+    return (round(cap_ret_pred[0][cyc_num-1],2),round(cap_ret[0][cyc_num-1]*rel_err,2))
 
 def cap_ret_offline_NN(eis_data,cycling_data):
     '''Offline sklearn MLPRegressor NN for prediction of capacity at every cycle 1-300n
@@ -463,5 +476,125 @@ def cap_ret_offline_NN(eis_data,cycling_data):
     MSE_by_cycle_offline_NN = joblib.load(model_MSE_by_cycle_filename)
     
     return cap_ret_pred, MSE_by_cycle_offline_NN
+
+def rdf_online_plot(eis_data,cycling_data,n_estimators, max_features, max_depth,test_size,cyc_num):
+    ret=loadtxt(os.path.join(data_path,'44_cell_cap_ret_300_cycles.csv'),delimiter=',',skiprows=1)
+    var=loadtxt(os.path.join(data_path,'44_cell_cap_diff_var_1st_10cycles.csv'),delimiter=',',skiprows=1)
+    Z1max=loadtxt(os.path.join(data_path,'Z1max.csv'), delimiter=',')
+    Z2max=loadtxt(os.path.join(data_path,'Z2max.csv'), delimiter=',')
+    f_max=loadtxt(os.path.join(data_path,'freq_fit_max.csv'), delimiter=',')
+    Z1_hf=loadtxt(os.path.join(data_path,'Z1_high_f.csv'), delimiter=',')
+    Z2_hf=loadtxt(os.path.join(data_path,'Z2_high_f.csv'), delimiter=',')
+    X_features=np.vstack((var[:,1:45],f_max,Z1max,Z2max,Z1_hf,Z2_hf))
+    X=np.transpose(X_features)
+    y=np.transpose(ret)
+    y=y[1:45]
+    model = RandomForestRegressor(n_estimators= n_estimators, max_features = max_features, max_depth=max_depth,random_state=42,n_jobs=-1)
+      
+    X_dev, X_test, y_dev, y_test = train_test_split(X, y, test_size= test_size, random_state=42)
+    model.fit(X_dev, y_dev)
+    
+    r2=model.score(X_test,y_test)
+    features= ml_features(eis_data,cycling_data)
+    
+    
+    cap_ret=model.predict(features)
+    err=average_err_per_cyc(y_test,model.predict(X_test))
+    fig, ax = plt.subplots(figsize=(8,6))
+    fig.text(0.6,0.8,'Model $R^{2}$ = '+str(round(r2,2)))
+    plt.plot(cap_ret[0],label='Prediction')
+    plt.plot(cap_ret[0]-err,'r',label='Error Range')
+    plt.plot(cap_ret[0]+err,'r')
+    plt.xlabel('Cycle Number')
+    plt.ylabel('Capacity Retetion, [%]')
+    plt.legend(loc='lower left')
+    plt.show()
+    
+    return()
+def cap_ret_online_NN_plot(eis_data,cycling_data, hidden_layers_sizes, learning_rate, alpha, n_iter, test_size,cyc_num,random_state=None):
+    
+    '''Online sklearn MLPRegressor NN for prediction of capacity at every cycle 1-300n
+    
+    Parameters
+    ----------
+    input_features : EIS and 1st 10 cycles experimental data
+    
+    hidden_layers_sizes: tuple of (N1, N2, N3 ... Nn) for n hidden layers with each layer having Nn nodes
+    
+    learning_rate : initial learning rate
+    
+    alpha: value for alpha
+    
+    n_iter: max iterations for NN learning
+    
+    test_size: fraction of training data set aside for test
+    
+    random_state: set to int for reproducible results. Default is None
+    
+
+    Returns
+    -------
+    cap_ret_pred: Predicted capacity retentions for experimental sample at each cycle
+    
+    r2_online_NN: R-squared value for predictions on test data vs. known values
+    
+    MSE_by_cycle_online_NN: MSE at each cycle number for test data vs. known values 
+    '''
+    
+    #Defining model
+    input_features=ml_features(eis_data,cycling_data)
+    online_NN  = MLPRegressor(solver='lbfgs', activation='tanh', max_iter= n_iter,
+                            learning_rate_init=learning_rate,
+                            hidden_layer_sizes = hidden_layers_sizes,
+                            alpha=alpha)
+    
+    #Getting Features and y_values
+    ret=loadtxt(os.path.join(data_path,'44_cell_cap_ret_300_cycles.csv'),delimiter=',',skiprows=1)
+    var=loadtxt(os.path.join(data_path,'44_cell_cap_diff_var_1st_10cycles.csv'),delimiter=',',skiprows=1)
+    Z1max=loadtxt(os.path.join(data_path,'Z1max.csv'), delimiter=',')
+    Z2max=loadtxt(os.path.join(data_path,'Z2max.csv'), delimiter=',')
+    Z1min=loadtxt(os.path.join(data_path,'Z1min.csv'), delimiter=',')
+    Z2min=loadtxt(os.path.join(data_path,'Z2min.csv'), delimiter=',')
+    f_max=loadtxt(os.path.join(data_path,'freq_fit_max.csv'), delimiter=',')
+    Z1_hf=loadtxt(os.path.join(data_path,'Z1_high_f.csv'), delimiter=',')
+    Z2_hf=loadtxt(os.path.join(data_path,'Z2_high_f.csv'), delimiter=',')               
+    
+    X_train_i=np.vstack((var[:,1:45],f_max,Z1max,Z2max,Z1_hf,Z2_hf))
+    X_train_i=np.transpose(X_train_i)
+    y_train_i=np.transpose(ret)
+    y_train_i=y_train_i[1:,0:300]
+
+    X_train, X_test, y_train, y_test = train_test_split(X_train_i, y_train_i, test_size= test_size, random_state=random_state)
+    
+    #Training model
+    
+    online_NN.fit(X_train, y_train)
+    
+    #Predicting test data
+    
+    y_pred_online = online_NN.predict(X_test)
+    
+    #Accuracy params for test data
+    
+    
+    r2_online_NN = r2_score(np.transpose(y_test), np.transpose(y_pred_online))
+    MSE_by_cycle_online_NN = MSE_per_cycle(y_test, y_pred_online)
+    
+    #Predict from given data
+    
+    cap_ret_pred = online_NN.predict(input_features)
+    err=average_err_per_cyc(y_test,model.predict(X_test))
+    fig, ax = plt.subplots(figsize=(8,6))
+    fig.text(0.6,0.8,'Model $R^{2}$ = '+str(round(r2,2)))
+    plt.plot(cap_ret[0],label='Prediction')
+    plt.plot(cap_ret[0]-err,'r',label='Error Range')
+    plt.plot(cap_ret[0]+err,'r')
+    plt.xlabel('Cycle Number')
+    plt.ylabel('Capacity Retetion, [%]')
+    plt.legend(loc='lower left')
+    plt.show()
+    return ()
+
+
     
     
\ No newline at end of file
